# Generated by Django 3.1.14 on 2024-05-22 16:19

from django.db import (
    migrations,
)


def add_exporting_data_sub_stage_status(apps, schema_editor):
    """Добавление нового статуса этапу выгрузки данных."""
    ExportingDataSubStageStatus = apps.get_model('edu_rdm_integration', 'ExportingDataSubStageStatus')  # noqa: N806
    ExportingDataSubStageStatus.objects.get_or_create(
        key='READY_FOR_EXPORT',
        title='Готов к выгрузке'
    )


class Migration(migrations.Migration):

    dependencies = [
        ('edu_rdm_integration', '0008_transferredentity'),
    ]

    operations = [
        migrations.RunPython(add_exporting_data_sub_stage_status, reverse_code=migrations.RunPython.noop),
    ]
